from meiga import Result, Error, Success, Failure, isSuccess
from meiga.decorators import meiga

from petisco.security.token_decoder.invalid_token_error import InvalidTokenError
from petisco.security.token_manager.interface_token_manager import ITokenManager
from petisco.domain.aggregate_roots.info_id import InfoId
from petisco.security.token_decoder.interface_token_decoder import ITokenDecoder
from petisco.security.token_decoder.token_decoder import TokenDecoder


class TokenManager(ITokenManager):
    def __init__(
        self,
        token_type: str,
        require_user: bool = False,
        token_decoder: ITokenDecoder = TokenDecoder(),
    ):
        self.token_type = token_type
        self.require_user = require_user
        self.token_decoder = token_decoder

    @meiga
    def execute(self, headers: dict) -> Result[InfoId, Error]:
        auth_token = headers.get("Authorization")

        if not auth_token:
            return Failure(
                InvalidTokenError(
                    message=f"This entry point expects a valid {self.token_type} Token"
                )
            )

        token = self.token_decoder.execute(auth_token).unwrap_or_return()

        self.ensure_required_token(token).unwrap_or_return()

        info_id = InfoId.from_token(token).update_from_headers(headers)

        return Success(info_id)

    def ensure_required_token(self, token) -> Result[bool, Error]:
        if (
            token.token_type != self.token_type
            or (self.require_user and not token.user_id)
            or (not self.require_user and token.user_id)
        ):
            return Failure(
                InvalidTokenError(
                    message=f"This entry point expects a valid {self.token_type} Token"
                )
            )
        return isSuccess



from meiga import Result, Error, Success

from petisco.security.token_manager.interface_token_manager import ITokenManager
from petisco.domain.aggregate_roots.info_id import InfoId


class NotImplementedTokenManager(ITokenManager):
    def execute(self, headers: dict) -> Result[InfoId, Error]:
        return Success(InfoId.from_headers(headers))



from typing import Any

from dataclasses import dataclass
from dataclasses import field
from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class Response(object):
    status_code: int
    content: None = field(default=None)
    headers: Any = None



import inspect
import re

from meiga import Error

from petisco.domain.errors.length_limit_string_value_object_error import (
    ExceedLengthLimitValueObjectError,
    NotReachMinimumValueObjectError,
)
from petisco.domain.value_objects.value_object import ValueObject


class InvalidStringValueObjectError(Error):
    def __init__(self, message):
        self.message = message


class StringValueObject(ValueObject):
    def __init__(self, value: str):
        self.value = value
        super(StringValueObject, self).__init__()

    def __hash__(self):
        return hash(str(self))

    def __repr__(self):
        return f"[{self.__class__.__name__}: {self.value}]"

    def __eq__(self, other):
        if issubclass(other.__class__, self.__class__) or issubclass(
            self.__class__, other.__class__
        ):
            return self.value == other.value
        else:
            return False

    def _raise_error(self, raise_cls):
        if "message" in inspect.getfullargspec(raise_cls).args:
            raise raise_cls(message=self.value)
        else:
            raise raise_cls()

    def _ensure_value_contains_valid_char(
        self, raise_cls=InvalidStringValueObjectError
    ):
        if not isinstance(self.value, str) or not re.search(
            r"^[\w]*(([',. -][\s]?[\w]?)?[\w]*)*$", self.value
        ):
            self._raise_error(raise_cls)

    def _ensure_value_is_less_than_n_char(
        self, max_num_chars: int, raise_cls=ExceedLengthLimitValueObjectError
    ):
        if len(self.value) > max_num_chars:
            self._raise_error(raise_cls)

    def _ensure_value_is_greater_than_n_char(
        self, max_num_chars: int, raise_cls=NotReachMinimumValueObjectError
    ):
        if len(self.value) < max_num_chars:
            self._raise_error(raise_cls)



import pytest

from typing import Dict

from petisco import ROOT_PATH, ClientId, UserId
from petisco.application.config.config import Config
from petisco.application.petisco import Petisco
from petisco.security.token_manager.jwt_token_builder import JwtTokenBuilder


@pytest.fixture
def given_petisco_version():
    return open(f"{ROOT_PATH}/VERSION", "r").read()[:-1]


@pytest.fixture
def given_headers_provider():
    def _given_headers_provider(headers: Dict):
        return lambda: headers

    return _given_headers_provider


@pytest.fixture
def given_any_headers_provider_with_correlation_id(
    given_any_header_with_valid_token,
    given_headers_provider,
    given_info_id_only_correlation_id,
):
    headers_token = given_any_header_with_valid_token
    headers_correlation = given_info_id_only_correlation_id.get_http_headers()
    return given_headers_provider({**headers_token, **headers_correlation})


@pytest.fixture
def given_any_headers_provider_with_info_id(
    given_any_header_with_valid_token, given_headers_provider, given_any_info_id
):
    headers_token = given_any_header_with_valid_token
    headers_correlation = given_any_info_id.get_http_headers()
    return given_headers_provider({**headers_token, **headers_correlation})


@pytest.fixture
def given_any_headers_provider(
    given_any_header_with_valid_token, given_headers_provider
):
    return given_headers_provider(given_any_header_with_valid_token)


@pytest.fixture
def given_any_headers_provider_with_user(
    given_any_header_with_valid_token_with_user_id, given_headers_provider
):
    return given_headers_provider(given_any_header_with_valid_token_with_user_id)


@pytest.fixture
def given_dummy_key():
    return (
        "-----BEGIN PRIVATE KEY-----\nMIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQC9b6fIsGAjrg7X"
        "\ncwnQWHUA1R1DCbB2oMQRmfrUVe2i3bIdjJ3ATgAcEAr+LDYas1tLM/zh/wLBYFzT\nCKfdFhn6YNfMHbbjcj"
        "+GfPM0kCnNtomsMUlDvHhqwLMEg8Af8r0cWorBWVrzM7Ab\nl1c9lhGR64U5oUCSSC27m+JxJFzc"
        "/FRe1dH4Dad1G9qx25DdZJoCuZfIqdYMGbt1\nzSaqXa9I4RTVPtzp4cYUG4iZ42+Gcn"
        "/dNrtgjXA5EPP3aGDoVNZWnlF4r8SSEQ3z\nMvNpRDtbGSrw5rKGYfW21da+7sqhvU/Q0DOeURAY4YuOaw+/Y3BoU"
        "/EFPcrZ344u\nRe5v5iwVAgMBAAECggEADYuERbdGyIZTjVguZXVNuGeElhyFP5YzLscBxlj2iuSr\nLa22Qd/R5i"
        "/HeXET1UjpyDDJbidbvRP0zyoZRpej9+EgzPl68DxlCmq4wRNlyc25"
        "\nflRv1qYNPiTVKAd7n00t9G8M6NJIzIbvmqlxlXe7feGHPx2mjk8W5gk5Ybml4i8Q"
        "\n5wfRAwCLmitLwuUlpNX5lcDPudAcOwO7ik854y1LwMs3necpQ4qyjLUFIeMxoq1u"
        "\nJCUn3ReCKwpTGcKNcPM6tf9yhhnZi1Lntytcdli9TqsJK0dRnczVuc6W+hCX/Szu\n9x57waLc+6w1i87aOn/XqzoS"
        "+R02Ct3emM1hbAd5gQKBgQD41sfsbM6inxQTvdF0\n1p2pF1Fw3VyymmL3dNMUNzD2GgbiLISGtzfvaIFLJn/OLRp"
        "+9XBIjtkFcpdsKktD\ngIjGf3DXI/hFV7doj1jB7udAhyegt7XZWo3JXcPNTlTXXf/dVGptIsfjMeo+9UEn\nZuIueJki"
        "/SEjtnZposBFS+efUQKBgQDC4z+bv2NeuCcjmMGjuvhp4xTQWY4lkWbd\ndaBbhh8fxlhfZSlkXQ8efNyuDMZR0EV"
        "+XGVQf1sU3Zx8iuuNwOsAXhtr0uTALKr9\nO3nYOik1bKwq7hn2uFCJ8zisNDQBlypflK8Kw7lTcTnh2UdMBxnjCfdvIK+L7Yyu"
        "\nnXE3bnY3hQKBgBpZxDoJG6ik599fTnrs7y0kxv8IsfRH3QsPV/rgDVNCVlBFHpOV\ne1ri2i43GXmf1Rp0qztDxT"
        "+KbvaJVAcF6XX7sNtXd8cADvAHngKLZfmzVkLysVMe\n/7y/LSjkjp9v2NR7R/neoQQyGPgpyIh/oJwDMVQ/Me8yNz"
        "/geDPeYdzBAoGARZWu\nWIX1jRRjpSoducBAgk46WEohnvhTqi8dmpBA/K9tTmJobRVtGI2sndih1wRuS7fh"
        "\n1OQkOUcHYNCZWnvPbMotMEqUsqlARDrAPpxFqnWq44kwbA9Tcm4Nhi79grPexVHp"
        "\nTgakXU7IRAO4mUTP0Wj1mBArneKroI1Gp2wheaECgYEA7rakdIlHHE3DnVeFnMd0\ngYrzBFRi"
        "+8IXssPRuFvou93tkFI7huEzz46SL1Igx9G/q5enA/1GSzKlIDBbebXx\nGzF/urX+jE+TaR3U8LUaV8dMFk3yF+bR2nnJabv"
        "+DMvyxJtUYiEnFgCypbG5m89Y\n9FHLq/CZ1uLCQhY1ITq/kTM=\n-----END PRIVATE KEY----- "
    )


@pytest.fixture
def given_auth_token_headers_creator(given_dummy_key):
    def _given_auth_token_headers_creator(
        type_token=str,
        client_id: ClientId = ClientId("client-id"),
        user_id: UserId = None,
    ):
        headers = {
            "Authorization": f"Bearer {JwtTokenBuilder.build(given_dummy_key, type_token, client_id, user_id=user_id)}"
        }
        return headers

    return _given_auth_token_headers_creator


@pytest.fixture
def given_any_header_with_valid_token(
    given_auth_token_headers_creator, given_any_token_type, given_any_client_id
):
    return given_auth_token_headers_creator(
        type_token=given_any_token_type, client_id=given_any_client_id
    )


@pytest.fixture
def given_any_header_with_valid_token_with_user_id(
    given_auth_token_headers_creator,
    given_any_token_type,
    given_any_client_id,
    given_any_user_id,
):
    return given_auth_token_headers_creator(
        type_token=given_any_token_type,
        client_id=given_any_client_id,
        user_id=given_any_user_id,
    )


@pytest.fixture
def given_any_petisco():
    return Petisco(config=Config(app_name="petisco", app_version="1.0.0"))



import jwt

from petisco.domain.value_objects.client_id import ClientId
from petisco.domain.value_objects.user_id import UserId


class JwtTokenBuilder:
    @staticmethod
    def build(
        key: str,
        token_type,
        client_id: ClientId,
        issuer="issuer-petisco",
        user_id: UserId = None,
    ):
        payload = {
            "iss": issuer,
            "typ": token_type,
            "exp": 1562338273,
            "iat": 1562334673,
            "cli": client_id.value,
            "sub": user_id.value if user_id else None,
        }
        token = jwt.encode(payload, key, algorithm="RS256").decode("utf-8")
        return token



from abc import ABCMeta, abstractmethod
from meiga import Result


class UseCase:

    __metaclass__ = ABCMeta

    @abstractmethod
    def execute(self, *args, **kwargs) -> Result:
        raise Result(failure=NotImplementedError())



from petisco.domain.value_objects.string_value_object import StringValueObject
from petisco.domain.errors.given_input_is_not_valid_error import (
    GivenInputIsNotValidError,
)
from petisco.domain.errors.length_limit_string_value_object_error import (
    ExceedLengthLimitValueObjectError,
)


class ClientId(StringValueObject):
    def guard(self):
        self._ensure_is_less_than_50_chars()
        self._ensure_value_contains_valid_char(raise_cls=GivenInputIsNotValidError)

    def _ensure_is_less_than_50_chars(self):
        if len(self.value) > 50:
            raise ExceedLengthLimitValueObjectError(message=self.value)
